/***************************************************************************************************
 *
 *  Ownage Mutator. Converted from UT2004 to UT99 by Zeropoint.
 *
 *  $CLASS        OwnageMutator
 *  $VERSION      1.00 (11-8-2008 13:15)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The ownage mutator.
 *
 **************************************************************************************************/
class OwnageMutator extends mutator config(Ownage);

#exec AUDIO IMPORT NAME=ownage FILE=Resources\ownage.wav

struct ClanTagInformation {                       // Clan tag description struct.
	var string tagString;                         // The tag of the clan.
	var bool bIsEndTag;                           // Whether the tag is placed at the end.
};

var config ClanTagInformation clanTags[10];       // The clan tags to recognize.
var config bool bPlayOwnageSound;                 // Play the 'Ownage!' sound.
var config string message;                        // Message to display.
var config bool bCenterMessage;                   // Center the message.
var Color centerMessageColor;                     // The color of the centered message.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the mutator.
 *
 **************************************************************************************************/
function preBeginPlay() {
	local OwnageMutator m;
	
	// Check for another instance of the mutator.
	foreach AllActors(Class'OwnageMutator', m) {
		// Multiple instances are not allowed.
		if (m != self) {
			destroy();
			return;
		}
	}
	
	super.preBeginPlay();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a player attempts to login to the server. Allows mutators to modify
 *                some of the login parameters.
 *  $PARAM        spawnClass  The PlayerPawn class to use for the player.
 *  $PARAM        portal      Name of the portal where the player wishes to spawn.
 *  $PARAM        option      Login option parameters.
 *
 **************************************************************************************************/
function modifyLogin(out class<playerpawn> spawnClass, out string portal, out string options) {
	local string playerName;
	local int index;
	local bool bFound;
	local pawn aPawn;
	local string msgToSend;
	
	// First let the other mutators modify the login as they might change the name.
	if (nextMutator != none) {
		nextMutator.modifyLogin(spawnClass, portal, options);
	}
	
	// Get player name.
	playerName = level.game.parseOption(options, "name");
	
	// Check if a matching clan tag is found.
	index = 0;
	while (!bFound && index < arrayCount(clanTags)) {
		// Does the current clan tag occur in the name of the player?
		if (clanTags[index].tagString != "" &&
		    len(playerName) > len(clanTags[index].tagString) &&
		    (clanTags[index].bIsEndTag && caps(right(playerName, len(clanTags[index].tagString))) == caps(clanTags[index].tagString) ||
		     !clanTags[index].bIsEndTag && caps(left(playerName, len(clanTags[index].tagString))) == caps(clanTags[index].tagString))) {
			// Yeah it does, OWNAGE!
			bFound = true;
		} else {
			// No, maybe the next one does...
			index++;
		}
	}
	
	// Show message & play sound if the player is a clan member.
	if (bFound) {
		if (message != "") {
			msgToSend = message;
			msgToSend = replace(msgToSend, "%playername%", playerName);
			msgToSend = replace(msgToSend, "%tag%", clanTags[index].tagString);
			if (bCenterMessage) {
				for (aPawn = level.pawnList; aPawn != none; aPawn = aPawn.nextPawn) {
					if (aPawn.isA('PlayerPawn')) {
						PlayerPawn(aPawn).clearProgressMessages();
						PlayerPawn(aPawn).setProgressTime(6);
						PlayerPawn(aPawn).setProgressMessage(msgToSend, 0);
						PlayerPawn(aPawn).setProgressColor(centerMessageColor, 0);
					}
				}
			} else {
				broadcastMessage(msgToSend, true);
			}
		}
		if (bPlayOwnageSound) {
			for (aPawn = level.pawnList; aPawn != none; aPawn = aPawn.nextPawn) {
				if (aPawn.isA('PlayerPawn')) {
					PlayerPawn(aPawn).clientReliablePlaySound(Sound'Ownage');
				}
			}
		}
	}

}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replaces a specified substring in a string with another substring.
 *  $PARAM        source  The original string, which is to be filtered.
 *  $PARAM        oldStr  Substring in the original string that is to be replaced.
 *  $PARAM        newStr  Replacement for the substring to be replaced.
 *  $RETURN       The specified string where all occurrences of oldStr are replaced with newStr.
 *
 **************************************************************************************************/
static function string replace(coerce string source, coerce string oldStr, coerce string newStr) {
	local bool bDone;
	local int subStrIndex;
	local string result;
	local string strLeft;
	
	strLeft = source;
	
	// Replace each occurrence of oldStr with newStr.
	while (!bDone) {
		
		// Find index of oldStr in the part not examined yet.
		subStrIndex = instr(strLeft, oldStr);
		
		// Update examined and unexamined parts.
		if (subStrIndex < 0) {
			bDone = true;
			result = result $ strLeft;
		} else {
			result = result $ left(strLeft, subStrIndex) $ newStr;
			strLeft = mid(strLeft, subStrIndex + len(oldStr));
		}
	}
	
	// Return the filtered string.
	return result;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	centerMessageColor=(r=255,g=0,b=0)
}